import { toJSON, fromJSON } from 'flatted';
import { OpenAPIV3 } from 'openapi-types';
import { OpenAPIFetcher } from './types';
export interface OpenAPIOperationData extends OpenAPICustomSpecProperties {
    path: string;
    method: string;
    /** Servers to be used for this operation */
    servers: OpenAPIV3.ServerObject[];
    /** Spec of the operation */
    operation: OpenAPIV3.OperationObject & OpenAPICustomOperationProperties;
    /** Securities that should be used for this operation */
    securities: [string, OpenAPIV3.SecuritySchemeObject][];
}
/**
 * Custom properties that can be defined at the entire spec level.
 */
export interface OpenAPICustomSpecProperties {
    /**
     * If `true`, code samples will not be displayed.
     * This option can be used to hide code samples for the entire spec.
     */
    'x-codeSamples'?: boolean;
    /**
     * If `true`, the "Try it" button will not be displayed.
     * This option can be used to hide code samples for the entire spec.
     */
    'x-hideTryItPanel'?: boolean;
}
/**
 * Custom properties that can be defined at the operation level.
 * These properties are not part of the OpenAPI spec.
 */
export interface OpenAPICustomOperationProperties {
    'x-code-samples'?: OpenAPICustomCodeSample[];
    'x-codeSamples'?: OpenAPICustomCodeSample[] | false;
    'x-custom-examples'?: OpenAPICustomCodeSample[];
    /**
     * If `true`, the "Try it" button will not be displayed.
     * https://redocly.com/docs/api-reference-docs/specification-extensions/x-hidetryitpanel/
     */
    'x-hideTryItPanel'?: boolean;
}
/**
 * Custom code samples that can be defined at the operation level.
 * It follows the spec defined by Redocly.
 * https://redocly.com/docs/api-reference-docs/specification-extensions/x-code-samples/
 */
export interface OpenAPICustomCodeSample {
    lang: string;
    label: string;
    source: string;
}
export { toJSON, fromJSON };
/**
 * Resolve an OpenAPI operation in a file and compile it to a more usable format.
 */
export declare function fetchOpenAPIOperation(input: {
    url: string;
    path: string;
    method: string;
}, rawFetcher: OpenAPIFetcher): Promise<OpenAPIOperationData | null>;
/**
 * Parse a raw string into an OpenAPI document.
 * It will also convert Swagger 2.0 to OpenAPI 3.0.
 * It can throw an `OpenAPIFetchError` if the document is invalid.
 */
export declare function parseOpenAPIV3(url: string, text: string): Promise<OpenAPIV3.Document>;
export declare class OpenAPIFetchError extends Error {
    readonly url: string;
    name: string;
    constructor(message: string, url: string);
}
